<?php

namespace Teknisa\COC\Controllers;

use Teknisa\GER\Utilities\Environment;
use Teknisa\GER\Services\CrudService;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use \Doctrine\ORM\EntityManager;

class CustoMenuDesign extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $entityManager;
    protected $dataSourceManager;
    protected $environment;
    protected $connection;
    protected $parameterBag;
    protected $dataSourceName = 'manualciam';
    protected $parameterBagColumns = array();
    protected $messagesProcedureInflightor;
    protected $fileserverPath;

    public function __construct($entityManager, Manager $dataSourceManager, Environment $environment, $connection, $parameterBag, $messagesProcedureInflightor, $fileserverPath) {
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->connection = $connection;
        $this->parameterBag = $parameterBag;
        $this->messagesProcedureInflightor = $messagesProcedureInflightor;
        $this->fileserverPath = $fileserverPath;
	}


    public function calculaCusto (DTO\Request\DataSet $request, DTO\Response $response){
    	try {
			foreach ($request->getDataSet()->getRows() as $row) {
				$connection = $this->entityManager->getConnection();

				$procedure = new StoredProcedure($connection, 'CUSTO_MANUALCIAM');
				$procedure->addParam(new Param('P_CDFILIAL',          Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
				$procedure->addParam(new Param('P_CDCIA',             Param::PARAM_INPUT, $row['CDCIA'], Param::PARAM_TYPE_STR));
				$procedure->addParam(new Param('P_CDMANCIA',          Param::PARAM_INPUT, $row['CDMANCIA'], Param::PARAM_TYPE_STR));
				$procedure->addParam(new Param('P_NRSERVICOCIAFILI',  Param::PARAM_INPUT, NULL));
				$procedure->addParam(new Param('P_CDCLASSE',          Param::PARAM_INPUT, NULL));
				$procedure->addParam(new Param('P_CDSUBGRU',          Param::PARAM_INPUT, NULL));
				$procedure->addParam(new Param('P_NRSEQSUBGRUSEMAM',  Param::PARAM_INPUT, NULL));
				$procedure->addParam(new Param('P_CDCICLO',           Param::PARAM_INPUT, NULL));
				$procedure->addParam(new Param('P_DATACUSTO',         Param::PARAM_INPUT, $row['DATACUSTO'], Param::PARAM_TYPE_STR));
				$procedure->addParam(new Param('P_TIPOCUSTO',         Param::PARAM_INPUT, $row['TIPOCUSTO'], Param::PARAM_TYPE_STR));
				$procedure->addParam(new Param('P_CDOPERADOR',        Param::PARAM_INPUT, $row['CDOPERADOR'], Param::PARAM_TYPE_STR));
				$procedure->addParam(new Param('P_NRORG',             Param::PARAM_INPUT, $row['NRORG'], Param::PARAM_TYPE_STR));
				$procedure->execute();
		    }
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 1));
		}
	}

    public function copiaCiclos (DTO\Request\DataSet $request, DTO\Response $response){
    	
    	try {
		    $connection = $this->entityManager->getConnection();
			foreach ($request->getDataSet()->getRows() as $row) {
		        $procedure = new StoredProcedure($connection, 'COPIA_COMPOSICAO_CICLO');

		        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['CDFILIALORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCIA', Param::PARAM_INPUT, $row['CDCIAORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDMANCIA', Param::PARAM_INPUT, $row['CDMANCIAORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRSERVICOCIAFILI', Param::PARAM_INPUT, $row['NRSERVICOCIAFILIORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCLASSE', Param::PARAM_INPUT, $row['CDCLASSEORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDSUBGRU', Param::PARAM_INPUT, $row['CDSUBGRUORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRSEQSUBGRUSEMAM', Param::PARAM_INPUT, $row['NRSEQSUBGRUSEMAMORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCICLOORIGEM', Param::PARAM_INPUT, $row['CDCICLOORIGEM'], Param::PARAM_TYPE_STR));

		        $procedure->addParam(new Param('P_CDFILIALDES', Param::PARAM_INPUT, $row['CDFILIALDES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCIADES', Param::PARAM_INPUT, $row['CDCIADES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDMANCIADES', Param::PARAM_INPUT, $row['CDMANCIADES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRSERVICOCIAFILIDES', Param::PARAM_INPUT, $row['NRSERVICOCIAFILIDES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCLASSEDES', Param::PARAM_INPUT, $row['CDCLASSEDES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDSUBGRUDES', Param::PARAM_INPUT, $row['CDSUBGRUDES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRSEQSUBGRUSEMAMDES', Param::PARAM_INPUT, $row['NRSEQSUBGRUSEMAMDES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCICLODESTINO', Param::PARAM_INPUT, $row['CDCICLODESTINO'], Param::PARAM_TYPE_STR));

		        // $procedure->addParam(new Param('P_IMAGEMDEFAULT', Param::PARAM_INPUT, $this->fileserverPath->getUndefinedImagePath(), Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_IMAGEMDEFAULT', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_COPIA_IMAGEM', Param::PARAM_INPUT, $row['COPIA_IMAGEM'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_SOBRESCREVER', Param::PARAM_INPUT, $row['SOBRESCREVER'], Param::PARAM_TYPE_STR));
				$procedure->addParam(new Param('P_COPIA_IT', Param::PARAM_INPUT, $row['COPIA_IT'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $this->environment->getNrOrg(), Param::PARAM_TYPE_STR));
		        $procedure->execute();
		    }
		} catch (\Exception $e) {
			$response->setError(new Error($this->messagesProcedureInflightor->convertMessage($e->getMessage()), 1));
		}
	}

	public function copiaMenu (DTO\Request\DataSet $request, DTO\Response $response){
		try {
			foreach ($request->getDataSet()->getRows() as $row) {

		        $connection = $this->entityManager->getConnection();
		        $procedure = new StoredProcedure($connection, 'COPIA_MENU_GRID');
		        $procedure->addParam(new Param('P_CDFILIPROC', Param::PARAM_INPUT, $row['P_CDFILIPROC'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDFILIALORI', Param::PARAM_INPUT, $row['P_CDFILIALORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCIAORI', Param::PARAM_INPUT, $row['P_CDCIAORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDMANCIAORI', Param::PARAM_INPUT, $row['P_CDMANCIAORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDFILIALDES', Param::PARAM_INPUT, $row['P_CDFILIALDES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCIADES', Param::PARAM_INPUT, $row['P_CDCIADES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_DTINIVGMADES', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_DTFIMVGMADES', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
		        // $procedure->addParam(new Param('P_IMAGEMDEFAULT', Param::PARAM_INPUT, $row['P_IMAGEMDEFAULT'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_IMAGEMDEFAULT', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_COPIA_IMAGEM', Param::PARAM_INPUT, $row['P_COPIA_IMAGEM'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_COPIA_PRODCIA', Param::PARAM_INPUT, $row['COPIA_PRODCIA'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_COPIA_IT', Param::PARAM_INPUT, $row['COPIA_IT'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $row['CDOPERADOR'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $row['NRORG'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRLISTAERROS', Param::PARAM_OUTPUT, null, null, 20));
		        $resultado = $procedure->execute();
		        if($resultado['P_NRLISTAERROS'] != null) {
		        	$this->messagesProcedureInflightor->concatErrorMessages($row['P_CDFILIPROC'], $resultado['P_NRLISTAERROS'], $response);
		        }
		    }
		} catch (\Exception $e) {
			$response->setError(new Error($this->messagesProcedureInflightor->convertMessage($e->getMessage()), 1));
		}
	}

	public function copiaServico (DTO\Request\DataSet $request, DTO\Response $response){
		try {

			$rows = $request->getDataSet()->getRows();
			foreach ($rows as $row) {
		        $connection = $this->entityManager->getConnection();
		        $procedure = new StoredProcedure($connection, 'COPIA_SERV_MENU_GRID');
		        $procedure->addParam(new Param('P_CDFILIPROC', Param::PARAM_INPUT, $row['P_CDFILIPROC'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDFILIALORI', Param::PARAM_INPUT, $row['P_CDFILIALORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCIAORI', Param::PARAM_INPUT, $row['P_CDCIAORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDMANCIAORI', Param::PARAM_INPUT, $row['P_CDMANCIAORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRLISTASERVORI', Param::PARAM_INPUT, $row['P_NRLISTASERVORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRPROXLISTASUBGRU', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDFILIALDES', Param::PARAM_INPUT, $row['P_CDFILIALDES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCIADES', Param::PARAM_INPUT, $row['P_CDCIADES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDMANCIADES', Param::PARAM_INPUT, $row['P_CDMANCIADES'], Param::PARAM_TYPE_STR));
		        // $procedure->addParam(new Param('P_IMAGEMDEFAULT', Param::PARAM_INPUT, $row['P_IMAGEMDEFAULT'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_IMAGEMDEFAULT', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_COPIA_IMAGEM', Param::PARAM_INPUT, $row['P_COPIA_IMAGEM'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_SOBRESCREVER', Param::PARAM_INPUT, $row['P_SOBRESCREVER'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_COPIA_PRODCIA', Param::PARAM_INPUT, $row['COPIA_PRODCIA'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_COPIA_IT', Param::PARAM_INPUT, $row['COPIA_IT'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $row['P_CDOPERADOR'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $row['P_NRORG'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRLISTAERROS', Param::PARAM_OUTPUT, null, null, 20));
		        $resultado = $procedure->execute();

		        if($resultado['P_NRLISTAERROS'] != null) {
		        	$this->messagesProcedureInflightor->concatErrorMessages($row['P_CDFILIPROC'], $resultado['P_NRLISTAERROS'], $response);
		        }

				$params = array
				(
					"NRORG"  => $row['P_NRORG'],
					"NRLISTA" => $row['P_NRLISTASERVORI']
				);

				$sqlDelete = "DELETE FROM LISTA WHERE NRORG = :NRORG AND NRLISTA = :NRLISTA";

				$queryDelete = $this->entityManager->getConnection()->prepare($sqlDelete);
		        $queryDelete->execute($params);
		    }
		} catch (\Exception $e) {
			$response->setError(new Error($this->messagesProcedureInflightor->convertMessage($e->getMessage()), 1));
		}

	}

	public function salvaListaSubgru (DTO\Request\DataSet $request, DTO\Response $response){
		try {
			foreach ($request->getDataSet()->getRows() as $row) {
				$params = array
				(
					"NRLISTA" => $row['P_NRLISTASUBGRUORI'],
					"NRORG" => $row['P_NRORG'],
					"CDFILIALORI" => $row['P_CDFILIALORI'],
					"CDCIAORI" => $row['P_CDCIAORI'],
					"CDMANCIAORI" => $row['P_CDMANCIAORI'],
					"NRSERVICOCIAFILIORI" => $row['P_NRSERVICOCIAFILIORI'],
					"CDCLASSEORI" => $row['P_CDCLASSEORI'],
				);
				$sql = "INSERT INTO LISTA(NRLISTA, CDLISTA, NRORG) SELECT :NRLISTA, CDSUBGRU || '|' || TO_CHAR(NRSEQSUBGRUSEMAM), :NRORG FROM SUBGRUSEMAM WHERE CDFILIAL = :CDFILIALORI AND CDCIA = :CDCIAORI AND CDMANCIA = :CDMANCIAORI AND NRSERVICOCIAFILI = :NRSERVICOCIAFILIORI AND CDCLASSE = :CDCLASSEORI";

				$query = $this->entityManager->getConnection()->prepare($sql);
		        $query->execute($params);
	    	}
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 1));
		}
	}

	public function copiaSubgrupo (DTO\Request\DataSet $request, DTO\Response $response){

		try {
			foreach ($request->getDataSet()->getRows() as $row) {
				if ($row['P_TODOCICLO'] == true) {
					$ciclos = "";
					$params = array
					(
						"CDFILIAL" => $row['P_CDFILIALORI'],
						"CDCIA" => $row['P_CDCIAORI'],
						"CDMANCIA" => $row['P_CDMANCIAORI'],
						"NRSERVICOCIAFILI" => $row['P_NRSERVICOCIAFILIORI'],
						"CDCLASSE" => $row['P_CDCLASSEORI'],
						"NRORG" => $row['P_NRORG']
					);

					$sql = "SELECT DISTINCT IDCICLO, CS.CDCICLO, CS.NRORG, CS.CDFILIAL, CS.CDCIA, CS.CDMANCIA, CS.NRSERVICOCIAFILI, CS.CDCLASSE FROM CICLOSUBGRUSEMAM CS JOIN CICLO ON CICLO.CDCICLO = CS.CDCICLO WHERE CS.CDFILIAL = :CDFILIAL AND CS.CDCIA = :CDCIA AND CS.CDMANCIA = :CDMANCIA AND CS.NRSERVICOCIAFILI = :NRSERVICOCIAFILI AND CS.CDCLASSE = :CDCLASSE AND CS.NRORG = :NRORG";

					$query = $this->entityManager->getConnection()->prepare($sql);
			        $query->execute($params);
			        $result = $query->fetchAll();
			        foreach ($result as $rows) {
			        	$ciclos .= "_".$rows["CDCICLO"];
			        }
				} else {
					$ciclos = $row['P_LISTACICLOS'];
				}

		        $connection = $this->entityManager->getConnection();
		        $procedure = new StoredProcedure($connection, 'COPIA_SUBGRU_SERV_MENU_GRID');
		        $procedure->addParam(new Param('P_CDFILIPROC', Param::PARAM_INPUT, $row['P_CDFILIPROC'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDFILIALORI', Param::PARAM_INPUT, $row['P_CDFILIALORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCIAORI', Param::PARAM_INPUT, $row['P_CDCIAORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDMANCIAORI', Param::PARAM_INPUT, $row['P_CDMANCIAORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRSERVICOCIAFILIORI', Param::PARAM_INPUT, $row['P_NRSERVICOCIAFILIORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCLASSEORI', Param::PARAM_INPUT, $row['P_CDCLASSEORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRLISTASUBGRUORI', Param::PARAM_INPUT, $row['P_NRLISTASUBGRUORI'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_LISTACICLOS', Param::PARAM_INPUT, $ciclos, Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDFILIALDES', Param::PARAM_INPUT, $row['P_CDFILIALDES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCIADES', Param::PARAM_INPUT, $row['P_CDCIADES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDMANCIADES', Param::PARAM_INPUT, $row['P_CDMANCIADES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRSERVICOCIAFILIDES', Param::PARAM_INPUT, $row['P_NRSERVICOCIAFILIDES'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDCLASSEDES', Param::PARAM_INPUT, $row['P_CDCLASSEDES'], Param::PARAM_TYPE_STR));
		        // $procedure->addParam(new Param('P_IMAGEMDEFAULT', Param::PARAM_INPUT, $row['P_IMAGEMDEFAULT'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_IMAGEMDEFAULT', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_COPIA_IMAGEM', Param::PARAM_INPUT, $row['P_COPIA_IMAGEM'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_SOBRESCREVER', Param::PARAM_INPUT, $row['P_SOBRESCREVER'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_COPIA_PRODCIA', Param::PARAM_INPUT, $row['COPIA_PRODCIA'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_COPIA_IT', Param::PARAM_INPUT, $row['COPIA_IT'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $row['P_CDOPERADOR'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_DTINIVGSUMADES', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_DTFIMVGSUMADES', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_IDALTERAAPROVADO', Param::PARAM_INPUT, 'N', Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $row['P_NRORG'], Param::PARAM_TYPE_STR));
		        $procedure->addParam(new Param('P_NRLISTAERROS', Param::PARAM_OUTPUT, null, null, 20));
		        $resultado = $procedure->execute();

		        if($resultado['P_NRLISTAERROS'] != null) {
		        	$this->messagesProcedureInflightor->concatErrorMessages($row['P_CDFILIPROC'], $resultado['P_NRLISTAERROS'], $response);
		        }

				$params = array
				(
					"NRORG"  => $row['P_NRORG'],
					"NRLISTA" => $row['P_NRLISTASUBGRUORI']
				);

				$sqlDelete = "DELETE FROM LISTA WHERE NRORG = :NRORG AND NRLISTA = :NRLISTA";
				$queryDelete = $this->entityManager->getConnection()->prepare($sqlDelete);
		        $queryDelete->execute($params);
		    }
		} catch (\Exception $e) {
			$response->setError(new Error($this->messagesProcedureInflightor->convertMessage($e->getMessage()), 1));
		}
	}

	protected function populateParameterBag(FilterCriteria $filterCriteria) {
        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );

        return $newFilterCriteria;
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

}