<?php

namespace Teknisa\COC\Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\FilterCriteria;

class ProdutoOrcamentoController extends \Teknisa\GER\Controllers\TeknisaCrud{
    
    protected $environment;
    protected $dataSourceManager;
    protected $dataSourceName = 'prodorcainf';
    
    public function __construct($crudservice, $parameterbag, $environment, $dataSourceManager){
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
        parent::__construct($crudservice, $parameterbag);
    }
    
    public function saveProdOrcamento(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            
            $produtos = $request->getDataSet()->getRows();
            $this->verificaProdutosComOrcamento($produtos, $response);
            parent::save($request, $response);
            
            
        } catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    private function verificaProdutosComOrcamento($produtos, $response){
        $newFiltro = $this->montaFiltro($produtos);
        $filterCriteria = new FilterCriteria($this->dataSourceName, $newFiltro);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        if(!empty($dataSet->getRows())){
            throw new \Exception('Produtos selecionados já foram incluídos a está unidade.');
        }
    }
    
    private function montaFiltro($produtos){
        $newFiltro = array();
        $arrayProdutos = array();
        for($i = 0; $i < sizeof($produtos); $i++){
            array_push($arrayProdutos, $produtos[$i]['CDPRODUTO']);
        }
        $newFiltro = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $produtos[0]['CDFILIAL']
            ),
            array(
                'columnName' => 'CDPRODUTO',
                'operator' => 'IN',
                'value' => $arrayProdutos
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $this->environment->getNrOrgTrab()
            )
        );
        return $newFiltro;
    }
    
}